package com.benyun.core.utils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.benyun.core.constant.RedisContant;
import com.benyun.core.dao.WdInfoDao;
import com.benyun.core.entity.WdInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.GeoResult;
import org.springframework.data.geo.GeoResults;
import org.springframework.data.geo.Point;
import org.springframework.data.redis.connection.RedisGeoCommands;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.stream.Collectors;

@Component
public class WdRedisStoreage {
    @Autowired
    private  RedisTemplate redisTemplate;

    @Autowired
    public   WdInfoDao wdInfoDao;

    @Autowired
    @Qualifier("executor")
    private ExecutorService executorService;


    /***
     * 获取中心网点的周边标签
     * @param wdInfo 中心网点信息
     * @return
     */
    public  List<String> getWdTag(WdInfo wdInfo){
        List<String> tag = (List<String>) redisTemplate.boundHashOps(RedisContant.WD_TAG).get(wdInfo.getWdId());
        if (tag != null)
            return tag;

        //分析该网点
        List<String> strings = wdTagAnalyse(wdInfo);
        return strings;
    }

    /***
     * 分析网点周边标签，并把分析结果存入redis（默认范围：1km）
     * @param wdInfo 中心网点信息
     * @return
     */
    public  List<String> wdTagAnalyse(WdInfo wdInfo){
        //1. 找到距离中心网点半径1km的所有网点Id
        Distance distance = new Distance(1, RedisGeoCommands.DistanceUnit.KILOMETERS);
        GeoResults<RedisGeoCommands.GeoLocation<Object>> radius = redisTemplate.boundGeoOps(wdInfo.getAddrCode().substring(0, 4)).radius(wdInfo.getWdId(), distance);
        List<GeoResult<RedisGeoCommands.GeoLocation<Object>>> content = radius.getContent();
        List<String> aroundWdId = content.stream().map(item -> {
            String name = (String) item.getContent().getName();
            return name;
        }).collect(Collectors.toList());

        //2. 查询周边网点标签的Set集
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("type_name_by")
                .in("wd_id",aroundWdId)
                    .groupBy("type_name_by");
        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);

        //3. 打标签
        List<String> tag = new ArrayList<>();
        wdInfos.stream().forEach(item -> {
            if(item != null){
                String typeNameBy = item.getTypeNameBy();
                if(typeNameBy != null && !typeNameBy.equals("")){
                    String[] split = typeNameBy.split(":");
                    if (split.length > 0)
                        tag.add(split[split.length-1]);
                }
            }
        });
        redisTemplate.boundHashOps(RedisContant.WD_TAG).put(wdInfo.getWdId(),tag);
        return tag;
    }

    /**
     * 将网点数据库 导入 redis
     * */
    public void writeWdGeoRedis(){
        //将所有网点坐标导入redis
        //writeWdGeoRedis1();

        //给网点打标签并存入redis中
        writeWdGeoRedis2();
    }

    private void writeWdGeoRedis1(){
        //1.将网点坐标存入redis中
        long start = System.currentTimeMillis();
        System.out.println("开始将网点坐标存入redis");
        for (WdInfo wdInfo : wdInfoDao.selectList(null)) {
            //开启线程
            CompletableFuture<Void> future = CompletableFuture.runAsync(()->{
                redisTemplate.boundGeoOps(wdInfo.getAddrCode().substring(0,4)).add(new Point(wdInfo.getLng().doubleValue(),wdInfo.getLat().doubleValue()),wdInfo.getWdId());
            },executorService);
            try {
                future.get();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            } catch (ExecutionException e) {
                throw new RuntimeException(e);
            }
        }
        long end = System.currentTimeMillis();
        System.out.println("完成存入 "+(end-start)/(1000*60));
    }

    private void writeWdGeoRedis2(){
        //2.对所有网点坐标进行分析打标签
        long start = System.currentTimeMillis();
        System.out.println("开始打标签");
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("wd_type_code", Arrays.asList("1","2","3","5"));
        List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);
        for (WdInfo wdInfo : wdInfos) {
            //开启线程
            CompletableFuture<Void> future = CompletableFuture.runAsync(()->{
                wdTagAnalyse(wdInfo);
            },executorService);
            try {
                future.get();
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            } catch (ExecutionException e) {
                throw new RuntimeException(e);
            }
        }
        long end = System.currentTimeMillis();
        System.out.println("完成打标签 "+(end-start)/(1000*60));
    }


}
